from .base import BaseChart
import json

from django.template.loader import render_to_string
from ..utils import JSONEncoderForHTML


class BaseHighCharts(BaseChart):
    def get_html_template(self):
        return "graphos/highcharts/html.html"

    def get_js_template(self):
        return "graphos/highcharts/js.html"

    def get_series(self):
        data = self.get_data()
        series_names = data[0][1:]
        serieses = []
        for i, name in enumerate(series_names):
            serieses.append({"name": name, "data": column(data, i+1)[1:]})
        return json.dumps(serieses, cls=JSONEncoderForHTML)

    def get_categories(self):
        return json.dumps(column(self.get_data(), 0)[1:], cls=JSONEncoderForHTML)

    def get_x_axis_title(self):
        return self.get_data()[0][0]


class LineChart(BaseHighCharts):
    def get_chart_type(self):
        return "line"


class BarChart(BaseHighCharts):
    def get_chart_type(self):
        return "bar"


class ColumnChart(BaseHighCharts):
    def get_chart_type(self):
        return "column"


class PieChart(BaseHighCharts):
    def get_series(self):
        data = self.get_data()
        series_names = data[0][1:]
        serieses = []
        for i, name in enumerate(series_names):
            serieses.append({"name": name, "data": pie_column(data, i+1)[1:]})
        return json.dumps(serieses)

    def get_chart_type(self):
        return "pie"

class AreaChart(BaseHighCharts):
    def get_chart_type(self):
        return "area"


def column(matrix, i):
    return [row[i] for row in matrix]


def pie_column(matrix, i):
    return [{'name':row[0],'y':row[1]} for row in matrix]



from django.test import TestCase

from pymongo.errors import CollectionInvalid

from .sources.base import BaseDataSource
from .sources.simple import SimpleDataSource
from .sources.csv_file import CSVDataSource
from .sources.model import ModelDataSource
from .sources.mongo import MongoDBDataSource

from .renderers import base, flot, gchart, yui, matplotlib_renderer
from .exceptions import GraphosException
from .utils import DEFAULT_HEIGHT, DEFAULT_WIDTH, get_default_options, get_db

from demo.models import Account

import os
import json

current_path = os.path.dirname(os.path.abspath(__file__))


class TestSources(TestCase):
    def test_base_data_source(self):
        data_source = BaseDataSource()
        self.assertTrue(hasattr(data_source, "get_data"))
        self.assertRaises(GraphosException, data_source.get_data,)
        self.assertTrue(hasattr(data_source, "get_header"))
        self.assertRaises(GraphosException, data_source.get_header)
        self.assertTrue(hasattr(data_source, "get_first_column"))
        self.assertRaises(GraphosException, data_source.get_first_column)

    def test_simple_data_source(self):
        data = [
            ['Year', 'Sales', 'Expenses'],
            ['2004', 1000, 400],
            ['2005', 1170, 460],
            ['2006', 660, 1120],
            ['2007', 1030, 540]
        ]
        data_source = SimpleDataSource(data)
        self.assertEqual(data_source.get_data(), data)
        self.assertEqual(data_source.get_header(),
                         ['Year', 'Sales', 'Expenses'])
        self.assertEqual(data_source.get_first_column(),
                         ['2004', '2005', '2006', '2007'])

    def test_csv_data_source(self):
        data = [
            ['Year', 'Sales', 'Expense'],
            ['2006', '1000', '400'],
            ['2007', '1170', '460'],
            ['2008', '660', '1120'],
            ['2009', '1030', '540']
        ]
        csv_file = open(os.path.join(current_path, "test_data/accounts.csv"),
                        "r")
        data_source = CSVDataSource(csv_file)
        self.assertEqual(data, data_source.get_data())
        self.assertEqual(data_source.get_header(),
                         ['Year', 'Sales', 'Expense'])
        self.assertEqual(data_source.get_first_column(),
                         ['2006', '2007', '2008', '2009'])

    def test_model_data_source(self):
        data = [
            ['year', 'sales', 'expenses'],
            [u'2004', 1000, 400],
            [u'2005', 1170, 460],
            [u'2006', 660, 1120],
            [u'2007', 1030, 540]
        ]
        #Create some rows
        Account.objects.create(year="2004", sales=1000,
                               expenses=400, ceo="Welch")
        Account.objects.create(year="2005", sales=1170,
                               expenses=460, ceo="Jobs")
        Account.objects.create(year="2006", sales=660,
                               expenses=1120, ceo="Page")
        Account.objects.create(year="2007", sales=1030,
                               expenses=540, ceo="Welch")
        query_set = Account.objects.all()
        data_source = ModelDataSource(query_set, ['year', 'sales', 'expenses'])
        self.assertEqual(data, data_source.get_data())
        self.assertEqual(data_source.get_header(),
                         ['year', 'sales', 'expenses'])
        self.assertEqual(data_source.get_first_column(),
                         ['2004', '2005', '2006', '2007'])


def get_mongodb_test_db(db_name, collection_name):
    cur_dir = os.path.dirname(os.path.realpath(__file__))
    test_data_file = open(cur_dir + '/test_data/mongodb/test_zips.json')
    db = get_db(db_name)
    try:
        db.create_collection(collection_name)
    except CollectionInvalid:
        pass
    for line in test_data_file:
        doc = json.loads(line)
        db[collection_name].save(doc)
    test_data_file.close()
    return db


class TestMongoDBSource(TestCase):

    def setUp(self):
        db_name = "test_db"
        collection_name = "zips"
        self.db = get_mongodb_test_db(db_name, collection_name)
        self.collection = self.db[collection_name]
        self.cursor = self.collection.find()
        self.fields = ['_id', 'pop']
        self.data = [['_id', 'pop'], ['35004', 6055], ['35005', 10616],
                     ['35006', 3205], ['35007', 14218], ['35010', 19942],
                     ['35014', 3062], ['35016', 13650], ['35019', 1781],
                     ['35020', 40549], ['35023', 39677], ['35031', 9058],
                     ['35033', 3448], ['35034', 3791], ['35035', 1282],
                     ['35040', 4675], ['35042', 4902], ['35043', 4781],
                     ['35044', 7985], ['35045', 13990], ['35049', '']]
        self.data_source = MongoDBDataSource(cursor=self.cursor,
                                             fields=self.fields)

    def test_data_source(self):
        self.assertTrue(hasattr(self.data_source, 'get_data'))
        self.assertTrue(hasattr(self.data_source, 'get_header'))
        self.assertTrue(hasattr(self.data_source, 'get_first_column'))
        self.assertEqual(self.data, self.data_source.get_data())
        self.assertEqual(self.fields, self.data_source.get_header())
        self.assertEqual(
            [el[0] for el in self.data[1:]],
            self.data_source.get_first_column()
        )

    def tearDown(self):
        self.db.drop_collection(self.collection.name)


class TestBaseRenderer(TestCase):

    def setUp(self):
        data = [
            ['Year', 'Sales', 'Expenses'],
            [2004, 1000, 400],
            [2005, 1170, 460],
            [2006, 660, 1120],
            [2007, 1030, 540]
        ]

        options = {"title": "Sales and Expences Graph"}

        self.default_options = {'title': 'Chart'}
        self.empty_options = {}
        self.data_source = SimpleDataSource(data)
        self.data = data
        self.options = options
        self.html_id = 'base_chart'
        self.template = 'graphos/as_html.html'
        self.header = data[0]

    def test_base_chart(self):
        chart = base.BaseChart(data_source=self.data_source,
                               options=self.options,
                               html_id=self.html_id)
        empty_options_chart = base.BaseChart(data_source=self.data_source,
                                             options=self.empty_options)
        self.assertTrue(hasattr(chart, "width"))
        self.assertEqual(DEFAULT_WIDTH, chart.width)
        self.assertTrue(hasattr(chart, "height"))
        self.assertEqual(DEFAULT_HEIGHT, chart.height)
        self.assertTrue(hasattr(chart, "header"))
        self.assertEqual(self.header, chart.header)
        self.assertTrue(hasattr(chart, "get_data"))
        self.assertEqual(self.data, chart.get_data())
        self.assertTrue(hasattr(chart, "get_data_json"))
        self.assertEqual(json.dumps(self.data), chart.get_data_json())
        self.assertTrue(hasattr(chart, "get_options"))
        self.assertEqual(self.options, chart.get_options())
        self.assertEqual(self.default_options,
                         empty_options_chart.get_options())
        self.assertTrue(hasattr(chart, "get_options_json"))
        self.assertEqual(json.dumps(self.options),
                         chart.get_options_json())
        self.assertTrue(hasattr(chart, "get_template"))
        self.assertEqual(self.template, chart.get_template())
        self.assertTrue(hasattr(chart, "get_html_template"))
        self.assertRaises(GraphosException, chart.get_html_template)
        self.assertTrue(hasattr(chart, "get_js_template"))
        self.assertRaises(GraphosException, chart.get_js_template)
        self.assertTrue(hasattr(chart, "get_html_id"))
        self.assertTrue(self.html_id, chart.get_html_id())
        self.assertTrue(hasattr(chart, "as_html"))


class TestFlotRenderer(TestCase):
    """ Test Cases for the graphos.renderers.flot module"""

    def setUp(self):
        data = [
            ['Year', 'Sales', 'Expenses'],
            [2004, 1000, 400],
            [2005, 1170, 460],
            [2006, 660, 1120],
            [2007, 1030, 540]
        ]
        self.data_source = SimpleDataSource(data)
        self.data = data
        self.options = {"title": "Sales and Expences Graph"}
        self.default_options = get_default_options()
        self.series_1 = [(2004, 1000), (2005, 1170), (2006, 660), (2007, 1030)]
        self.series_2 = [(2004, 400), (2005, 460), (2006, 1120), (2007, 540)]
        self.html_template = 'graphos/flot/html.html'
        self.js_template = 'graphos/flot/js.html'
        self.data_source = SimpleDataSource(data)
        self.data = data
        self.html_id = 'base_chart'
        self.header = data[0]
        series_object_1 = {'data': [(2004, 1000),
                                    (2005, 1170),
                                    (2006, 660),
                                    (2007, 1030)],
                           'label': 'Sales'}
        series_object_2 = {'data': [(2004, 400),
                                    (2005, 460),
                                    (2006, 1120),
                                    (2007, 540)],
                           'label': 'Expenses'}
        self.series_objects = [series_object_1, series_object_2]

    def test_base_flot_chart(self):
        chart = flot.BaseFlotChart(data_source=self.data_source,
                                   options=self.options)
        empty_options_chart = flot.BaseFlotChart(data_source=self.data_source,
                                                 options={})
        json_data = chart.get_serieses()
        self.assertEqual([self.series_1, self.series_2], json_data)
        self.assertEqual(self.html_template, chart.get_html_template())
        self.assertEqual(self.js_template, chart.get_js_template())
        default = get_default_options()
        self.assertEqual(default,
                         empty_options_chart.get_options())
        default.update(self.options)
        self.assertEqual(default, chart.get_options())
        self.assertEqual(json.dumps(default), chart.get_options_json())
        self.assertEqual(self.series_objects, chart.get_series_objects())
        self.assertEqual(json.dumps(self.series_objects),
                         chart.get_series_objects_json())

    def test_line_chart(self):
        chart = flot.LineChart(data_source=self.data_source,
                               options=self.options)
        empty_options_chart = flot.LineChart(data_source=self.data_source,
                                             options={})
        json_data = chart.get_serieses()
        self.assertEqual([self.series_1, self.series_2], json_data)
        default = get_default_options("lines")
        self.assertEqual(default,
                         empty_options_chart.get_options())
        default.update(self.options)
        self.assertEqual(default,
                         chart.get_options())

    def test_bar_chart(self):
        chart = flot.BarChart(data_source=self.data_source,
                              options=self.options)
        empty_options_chart = flot.BarChart(data_source=self.data_source,
                                            options={})
        json_data = chart.get_serieses()
        self.assertEqual([self.series_1, self.series_2], json_data)
        default = get_default_options("bars")
        self.assertEqual(default,
                         empty_options_chart.get_options())
        default.update(self.options)
        self.assertEqual(default,
                         chart.get_options())

    def test_point_chart(self):
        chart = flot.PointChart(data_source=self.data_source,
                                options=self.options)
        empty_options_chart = flot.PointChart(data_source=self.data_source,
                                              options={})
        json_data = chart.get_serieses()
        self.assertEqual([self.series_1, self.series_2], json_data)
        default = get_default_options("points")
        self.assertEqual(default,
                         empty_options_chart.get_options())
        default.update(self.options)
        self.assertEqual(default,
                         chart.get_options())


class TestGchartRenderer(TestCase):
    def setUp(self):
        data = [
            ['Year', 'Sales', 'Expenses'],
            [2004, 1000, 400],
            [2005, 1170, 460],
            [2006, 660, 1120],
            [2007, 1030, 540]
        ]
        self.data_source = SimpleDataSource(data)
        self.data = data

    def test_line_chart(self):
        chart = gchart.LineChart(data_source=self.data_source)
        self.assertNotEqual(chart.as_html(), "")
        self.assertTrue("LineChart" in chart.as_html())

    def test_column_chart(self):
        chart = gchart.ColumnChart(data_source=self.data_source)
        self.assertNotEqual(chart.as_html(), "")
        self.assertTrue("Column" in chart.as_html())

    def test_bar_chart(self):
        chart = gchart.BarChart(data_source=self.data_source)
        self.assertNotEqual(chart.as_html(), "")
        self.assertTrue("BarChart" in chart.as_html())

    def test_candlestick_chart(self):
        chart = gchart.CandlestickChart(data_source=self.data_source)
        self.assertNotEqual(chart.as_html(), "")
        self.assertTrue("CandlestickChart" in chart.as_html())

    def test_gauge_chart(self):
        chart = gchart.GaugeChart(data_source=self.data_source)
        self.assertNotEqual(chart.as_html(), "")
        self.assertTrue("Gauge" in chart.as_html())


class TestYUIRenderer(TestCase):
    def setUp(self):
        data = [
            ['Year', 'Sales', 'Expenses'],
            [2004, 1000, 400],
            [2005, 1170, 460],
            [2006, 660, 1120],
            [2007, 1030, 540]
        ]
        self.data_source = SimpleDataSource(data)
        self.data = data

    def test_line_chart(self):
        chart = yui.LineChart(data_source=self.data_source)
        self.assertNotEqual(chart.as_html(), "")
        self.assertTrue("line" in chart.as_html())


class TestMatplotlibRenderer(TestCase):
    def setUp(self):
        data = [['Year', 'Sales', 'Expenses', 'Items Sold', 'Net Profit'],
                ['2004', 1000, 400, 100, 600],
                ['2005', 1170, 460, 120, 310],
                ['2006', 660, 1120, 50, -460],
                ['2007', 1030, 540, 100, 200]]
        self.data_source = SimpleDataSource(data)
        self.data = data

    def test_line_chart(self):
        chart = matplotlib_renderer.LineChart(self.data_source)
        self.assertNotEqual(chart.as_html(), "")

    def test_bar_chart(self):
        chart = matplotlib_renderer.BarChart(self.data_source)
        self.assertNotEqual(chart.as_html(), "")



from django.views.generic.base import View
from django.http import HttpResponse

import json


class RendererAsJson(View):
    "A view which is used to get a chart as json."
    "Subclasses should create get_context_data, which populates a chart"
    def get(self, *args, **kwargs):
        context = self.get_context_data()
        chart = context["chart"]
        return HttpResponse(json.dumps({"chart_data": chart.get_data()}))

class FlotAsJson(View):
    "A view which is used to get a chart as json."
    "Subclasses should create get_context_data, which populates a chart"
    def get(self, *args, **kwargs):
        context = self.get_context_data()
        chart = context["chart"]
        label = chart.get_data()[0][1]
        return HttpResponse(json.dumps({"data": chart.get_data(), "label": label}))



from .simple import SimpleDataSource

import csv

class CSVDataSource(SimpleDataSource):
    "A data source for CSV files"
    def __init__(self, csv_file, fields=None):
        """csv_file: A file like object which should be charted
        """
        reader = csv.reader(csv_file)
        data =[row for row in reader]
        self.data = data
        self.fields = fields



import json
from .base import BaseChart


class BaseC3JS(BaseChart):
    """docstring for BaseC3JS"""
    def get_html_template(self):
        return "graphos/c3js/html.html"

    def get_js_template(self):
        return "graphos/c3js/js_base.html"

    def get_categories(self):
        return [x[0] for x in self.get_data()]

    def get_x_axis_title(self):
        return self.get_data()[0][0]

    def get_columns_data(self):
        return json.dumps(map(list, zip(*self.get_data())))



class LineChart(BaseC3JS):
    def get_chart_type(self):
        return "line"


class BarChart(BaseC3JS):
    def get_chart_type(self):
        return "bar"


class SplineChart(BaseC3JS):
    def get_chart_type(self):
        return "spline"


class PieChart(BaseC3JS):
    def get_js_template(self):
        return "graphos/c3js/js_pie.html"

    def get_data(self):
        _data = super(PieChart, self).get_data()
        return json.dumps(_data[1:])

    def get_chart_type(self):
        return "pie"


class ColumnChart(BaseC3JS):
    def get_js_template(self):
        return "graphos/c3js/js_column.html"

    """
    C3 doesn't have column type chart, so we have to render the column chart by
    rotating the axis in the js
    axis: {
        rotated: true
    }
    """
    def get_chart_type(self):
        return "column"


class DonutChart(BaseC3JS):
    def get_js_template(self):
        return "graphos/c3js/js_donut.html"

    def get_chart_type(self):
        return "donut"



#Named such to not clash with matplotlib
from .base import BaseChart

import matplotlib
matplotlib.use('Agg')  # http://stackoverflow.com/a/4706614/202168
import matplotlib.pyplot as plt
from matplotlib.ticker import FormatStrFormatter

try:
    # python 2
    from StringIO import StringIO
except ImportError:
    # python 3
    from io import BytesIO as StringIO

import base64


class BaseMatplotlibChart(BaseChart):

    def get_html_template(self):
        return "graphos/matplotlib_renderer/line_chart.html"

    def get_serieses(self):
        data_only = self.get_data()[1:]
        serieses = []
        for i in range(0, len(self.header)):
            current_column = [float(el[i]) for el in data_only]
            serieses.append(current_column)
        return serieses

    def render_js(self):
        return ""


class LineChart(BaseMatplotlibChart):

    def get_image(self):
        fig = plt.figure()
        ax = fig.add_subplot(111)
        serieses = self.get_serieses()
        for i in range(1, len(serieses)):
            ax.plot(serieses[0], serieses[i])
        ax.xaxis.set_major_formatter(FormatStrFormatter('%d'))
        out = StringIO()
        plt.savefig(out)
        out.seek(0)
        return "data:image/png;base64,%s" % base64.encodestring(out.read())


class BarChart(BaseMatplotlibChart):

    def get_image(self):
        fig = plt.figure()
        ax = fig.add_subplot(111)
        serieses = self.get_serieses()
        for i in range(1, len(serieses)):
            ax.bar(serieses[0], serieses[1], 0.35)
        ax.xaxis.set_major_formatter(FormatStrFormatter('%d'))
        out = StringIO()
        plt.savefig(out)
        out.seek(0)
        return "data:image/png;base64,%s" % base64.encodestring(out.read())



""" Model Plot Data Handler"""
from .simple import SimpleDataSource


def get_field_values(row, fields):
    data = []
    for field in fields:
        value = getattr(row, field)
        data.append(value if not callable(value) else value())
    return data


class ModelDataSource(SimpleDataSource):
    """
    Normalizes data contained in a queryset to format usable by renderers
    """
    def __init__(self, queryset, fields=None):
        """
        : param queryset: :type Django ORM queryset
        : param fields: :type  list of strings
        Example usage:
            queryset = Account.objects.all()
            mds = ModelDataSource(queryset, fields=['year', 'sales', 'expenses'])
            # This assumes the following model Account:
            class Account(models.Model):
                year = models.IntegerField()
                sales = models.DecimalField()
                expenses = models.DecimalField()
                profit = models.DecimalField()
        """
        self.queryset = queryset
        if fields:
            self.fields = fields
        else:
            self.fields = [el.name for el in self.queryset.model._meta.fields]
        self.data = self.create_data()

    def create_data(self):
        data = [self.fields]
        for row in self.queryset:
            data.append(get_field_values(row, self.fields))
        return data



import json
import sys

from django.template.loader import render_to_string
from ..exceptions import GraphosException
from ..utils import DEFAULT_HEIGHT, DEFAULT_WIDTH, get_random_string, JSONEncoderForHTML
from ..encoders import GraphosEncoder


class BaseChart(object):

    def __init__(self, data_source, html_id=None,
                 width=None, height=None,
                 options=None, encoder=GraphosEncoder,
                 *args, **kwargs):
        """
        : param data_source: :type graphos.sources.base.BaseDataSource subclass instance.
        : param html_id: :type string: Id of the div where you would like chart to be rendered
        : param width: :type integer: Width of the chart div
        : param height: :type integer: Height of the chart div
        """
        self.data_source = data_source
        self.html_id = html_id or get_random_string()
        self.height = height or DEFAULT_HEIGHT
        self.width = width or DEFAULT_WIDTH
        self.options = options or {}
        self.header = data_source.get_header()
        self.encoder = encoder
        self.context_data = kwargs

    def get_data(self):
        return self.data_source.get_data()

    def get_data_json(self):
        return json.dumps(self.get_data(), cls=JSONEncoderForHTML)

    def get_options(self):
        options = self.options
        if not 'title' in options:
            options['title'] = "Chart"
        return options

    def get_options_json(self):
        return json.dumps(self.get_options(), cls=JSONEncoderForHTML)

    def get_template(self):
        return 'graphos/as_html.html'

    def get_html_template(self):
        raise GraphosException("Not Implemented")

    def get_js_template(self):
        raise GraphosException("Not Implemented")

    def get_html_id(self):
        return self.html_id

    def get_context_data(self):
        return self.context_data

    def as_html(self):
        context = {
            'html': self.render_html(),
            'js': self.render_js(),
        }
        return render_to_string(self.get_template(), context)

    def render_html(self):
        context = {"chart": self}
        return render_to_string(self.get_html_template(), context)

    def render_js(self):
        context = {"chart": self}
        return render_to_string(self.get_js_template(), context)

    def zip_list(self, *args):
        rv = zip(*args)
        if sys.version_info < (3,0):
            return rv
        return list(rv)



import datetime
import decimal
import json
import uuid
import random
import string

from django.utils import six, timezone
from django.utils.encoding import force_text
from django.utils.functional import Promise
from django.db.models.query import QuerySet

DEFAULT_HEIGHT = 400
DEFAULT_WIDTH = 800

DB_HOST = ["localhost"]
DB_PORT = 27017


def get_random_string():
    random_letter = lambda: random.choice(string.ascii_letters)
    random_string = "".join([random_letter()
                             for el in range(10)])
    return random_string


def get_default_options(graph_type="lines"):
    """ default options """
    options = {"series": {"%s" % graph_type: {"show": "true"}},
               "legend": {"position": 'ne'},
               "title": "Chart"}
    return options


def get_db(db_name=None):
    """ GetDB - simple function to wrap getting a database
    connection from the connection pool.
    """
    import pymongo
    return pymongo.Connection(host=DB_HOST,
                              port=DB_PORT)[db_name]


class JSONEncoderForHTML(json.JSONEncoder):
    """An encoder that produces JSON safe to embed in HTML.
    To embed JSON content in, say, a script tag on a web page, the
    characters &, < and > should be escaped. They cannot be escaped
    with the usual entities (e.g. &amp;) because they are not expanded
    within <script> tags.
    """
    def default(self, obj):
        # Taken from https://github.com/tomchristie/django-rest-framework/blob/master/rest_framework/utils/encoders.py
        # For Date Time string spec, see ECMA 262
        # http://ecma-international.org/ecma-262/5.1/#sec-15.9.1.15
        if isinstance(obj, Promise):
            return force_text(obj)
        elif isinstance(obj, datetime.datetime):
            representation = obj.isoformat()
            if representation.endswith('+00:00'):
                representation = representation[:-6] + 'Z'
            return representation
        elif isinstance(obj, datetime.date):
            return obj.isoformat()
        elif isinstance(obj, datetime.time):
            if timezone and timezone.is_aware(obj):
                raise ValueError("JSON can't represent timezone-aware times.")
            representation = obj.isoformat()
            if obj.microsecond:
                representation = representation[:12]
            return representation
        elif isinstance(obj, decimal.Decimal):
            # Serializers will coerce decimals to strings by default.
            return float(obj)
        elif isinstance(obj, uuid.UUID):
            return six.text_type(obj)
        elif isinstance(obj, QuerySet):
            return tuple(obj)
        elif isinstance(obj, six.binary_type):
            # Best-effort for binary blobs. See #4187.
            return obj.decode('utf-8')
        elif hasattr(obj, 'tolist'):
            # Numpy arrays and array scalars.
            return obj.tolist()
        elif hasattr(obj, '__getitem__'):
            try:
                return dict(obj)
            except:
                pass
        elif hasattr(obj, '__iter__'):
            return tuple(item for item in obj)
        return super(JSONEncoderForHTML, self).default(obj)


    def encode(self, o):
        # Override JSONEncoder.encode because it has hacks for
        # performance that make things more complicated.
        chunks = self.iterencode(o, True)
        if self.ensure_ascii:
            return ''.join(chunks)
        else:
            return u''.join(chunks)

    def iterencode(self, o, _one_shot=False):
        try:
            chunks = super(JSONEncoderForHTML, self).iterencode(o, _one_shot)
        except TypeError:
            # for python 2.6 compatibility
            chunks = super(JSONEncoderForHTML, self).iterencode(o)
        for chunk in chunks:
            chunk = chunk.replace('&', '&amp;')
            chunk = chunk.replace('<', '&lt;')
            chunk = chunk.replace('>', '&gt;')
            yield chunk



""" Mongodb Plot Data Source """

from .simple import SimpleDataSource


def get_row(doc, fields):
    current_row = []
    for i in range(len(fields)):
        try:
            field = doc[fields[i]]
        except KeyError:
            field = ""
        current_row.append(field)
    return current_row


class MongoDBDataSource(SimpleDataSource):
    """ MongoDBDataSource """

    def __init__(self,
                 cursor,
                 fields,
                 *args,
                 **kwargs):

        self.cursor = cursor
        self.fields = fields
        self._doc_list = []

    def get_data(self):
        if self._doc_list:
            return self._doc_list
        doc_list = []
        doc_list.append(self.fields)
        for doc in self.cursor:
            current_row = get_row(doc, self.fields)
            doc_list.append(current_row)
        self._doc_list = doc_list
        return doc_list

    @property
    def data(self):
        return self.get_data()



from .base import BaseChart


class BaseGChart(BaseChart):
    def get_html_template(self):
        return "graphos/gchart/html.html"


class LineChart(BaseGChart):
    def get_js_template(self):
        return "graphos/gchart/line_chart.html"


class GaugeChart(BaseGChart):
    def get_js_template(self):
        return "graphos/gchart/gauge_chart.html"


class ColumnChart(BaseGChart):
    def get_js_template(self):
        return "graphos/gchart/column_chart.html"


class BarChart(BaseGChart):
    def get_js_template(self):
        return "graphos/gchart/bar_chart.html"

    def get_options(self):
        options = super(BarChart, self).get_options()
        if not 'vAxis' in options:
            vaxis = self.data_source.get_header()[0]
            options['vAxis'] = {'title': vaxis}
        return options


class CandlestickChart(BaseGChart):
    def get_js_template(self):
        return "graphos/gchart/candlestick_chart.html"


class PieChart(BaseGChart):
    def get_js_template(self):
        return "graphos/gchart/pie_chart.html"


class TreeMapChart(BaseGChart):
    def get_js_template(self):
        return "graphos/gchart/treemap_chart.html"


class AreaChart(BaseGChart):
    def get_js_template(self):
        return "graphos/gchart/area_chart.html"



from .base import BaseChart


class BaseYuiChart(BaseChart):
    def get_data(self):
        data = super(BaseYuiChart, self).get_data()
        header = self.header
        data_only = data[1:]
        rows = []
        for row in data_only:
            rows.append(dict(zip(header, row)))
        return rows

    def get_category_key(self):
        return self.data_source.get_header()[0]

    def get_html_template(self):
        return "graphos/yui/html.html"


class LineChart(BaseYuiChart):
    def get_js_template(self):
        return "graphos/yui/line_chart.html"

    def get_chart_type(self):
        return "line"


class SplineChart(BaseYuiChart):
    def get_js_template(self):
        return "graphos/yui/spline_chart.html"

    def get_chart_type(self):
        return "spline"


class BarChart(BaseYuiChart):
    def get_js_template(self):
        return "graphos/yui/bar_chart.html"

    def get_chart_type(self):
        return "bar"


class ColumnChart(BaseYuiChart):
    def get_js_template(self):
        return "graphos/yui/column_chart.html"

    def get_chart_type(self):
        return "column"


class PieChart(BaseYuiChart):
    def get_js_template(self):
        return "graphos/yui/pie_chart.html"

    def get_chart_type(self):
        return "pie"
    
    def get_data(self):
        # FIXME
        data = self.data_source.get_data()
        header = self.header
        header_first_two_columns = header[:2]
        data_only = data[1:]
        data_only_with_first_two_columns = [each[:2] for each in data_only]
        rows = []
        for row in data_only_with_first_two_columns:
            rows.append(dict(zip(header_first_two_columns, row)))
        return rows


class AreaChart(BaseYuiChart):
    def get_js_template(self):
        return "graphos/yui/area_chart.html"

    def get_chart_type(self):
        return "area"


class AreaSplineChart(BaseYuiChart):
    def get_js_template(self):
        return "graphos/yui/area_spline_chart.html"

    def get_chart_type(self):
        return "areaspline"


class ComboChart(BaseYuiChart):
    def get_js_template(self):
        return "graphos/yui/combo.html"

    def get_chart_type(self):
        return "combo"


class ComboSplineChart(BaseYuiChart):
    def get_js_template(self):
        return "graphos/yui/combo_spline.html"

    def get_chart_type(self):
        return "combospline"


class MarkerSeriesChart(BaseYuiChart):
    def get_js_template(self):
        return "graphos/yui/marker_series.html"

    def get_chart_type(self):
        return "markerseries"


import json

from .base import BaseChart
from ..utils import get_default_options, JSONEncoderForHTML


class BaseFlotChart(BaseChart):
    """ LineChart """

    def get_serieses(self):
        # Assuming self.data_source.data is:
        # [['Year', 'Sales', 'Expenses'], [2004, 100, 200], [2005, 300, 250]]
        data_only = self.get_data()[1:]
        # first_column = [2004, 2005]
        first_column = [el[0] for el in data_only]
        serieses = []
        for i in range(1, len(self.header)):
            current_column = [el[i] for el in data_only]
            current_series = self.zip_list(first_column, current_column)
            serieses.append(current_series)
        # serieses = [[(2004, 100), (2005, 300)], [(2004, 200), (2005, 250)]]
        return serieses

    def get_series_objects(self):
        series_objects = []
        serieses = self.get_serieses()
        for i in range(1, len(self.header)):
            series_object = {}
            series_object['label'] = self.header[i]
            series_object['data'] = serieses[i - 1]
            series_objects.append(series_object)
        # series_objects = [{'label': 'Sales', 'data': [(2004, 100), (2005, 300)]}, {'label': 'Expenses': 'data': [(2004, 100), (2005, 300)]}]
        return series_objects

    def get_series_pie_objects(self):
        series_objects = []
        serieses = self.get_data()[1:]
        try:
            for i in serieses:
                series_object = {}
                series_object['label'] = i[0]
                series_object['data'] = i[1]
                series_objects.append(series_object)
        except IndexError:
            print("Input Data Format is [['Year', 'Sales'], [2004, 100], [2005, 300]]")
        # series_objects = [{'label': '2004', 'data': 100}, {'label': '2005': 'data': 300}]
        return json.dumps(series_objects, cls=JSONEncoderForHTML)

    def get_series_objects_json(self):
        return json.dumps(self.get_series_objects(), cls=JSONEncoderForHTML)

    def get_options(self):
        options = get_default_options()
        options.update(self.options)
        return options

    def get_html_template(self):
        return 'graphos/flot/html.html'

    def get_js_template(self):
        return 'graphos/flot/js.html'


class PointChart(BaseFlotChart):

    def get_options(self):
        options = get_default_options("points")
        options.update(self.options)
        return options


class LineChart(BaseFlotChart):
    """ LineChart """

    def get_options(self):
        options = get_default_options("lines")
        options.update(self.options)
        return options


class BarChart(BaseFlotChart):

    def get_options(self):
        options = get_default_options("bars")
        options.update(self.options)
        return options


class ColumnChart(BaseFlotChart):

    def get_options(self):
        options = get_default_options("bars")
        options.update(self.options)
        options["horizontal"] = True
        return options


class PieChart(BaseFlotChart):

    def get_options(self):
        options = get_default_options("pie")
        options.update(self.options)
        return options

    def get_js_template(self):
        return 'graphos/flot/pie_chart.html'



from .base import BaseChart
import json

from ..utils import JSONEncoderForHTML

class BaseMorrisChart(BaseChart):

    def get_data(self):
        header = self.header
        data = super(BaseMorrisChart, self).get_data()
        data_only = data[1:]
        rows = []
        for row in data_only:
            rows.append(dict(zip(header, row)))
        return rows

    def get_category_key(self):
        return self.header[0]

    def get_y_keys(self):
        try:
            return json.dumps(self.options['ykeys'], cls=JSONEncoderForHTML)
        except KeyError:
            return json.dumps(self.header[1:], cls=JSONEncoderForHTML)

    def get_html_template(self):
        return "graphos/morris/html.html"

    def get_js_template(self):
        return "graphos/morris/chart.html"


class LineChart(BaseMorrisChart):
    def chart_type(self):
        return "Line"


class BarChart(BaseMorrisChart):
    def chart_type(self):
        return "Bar"


class DonutChart(BaseMorrisChart):
    def get_data(self):
        data = super(BaseMorrisChart, self).get_data()
        data_only = data[1:]
        return [{"label": el[0], "value": el[1]} for el in data_only]

    def chart_type(self):
        return "Donut"

    def get_js_template(self):
        return "graphos/morris/donut_chart.html"


class AreaChart(BaseMorrisChart):
    def chart_type(self):
        return "Area"



"""Base Plot Data Handler"""

from ..exceptions import GraphosException


class BaseDataSource(object):
    def __init__(*args, **kwargs):
        pass

    def get_data(self):
        "Get all the data. Subclasses should override this"
        raise GraphosException("Not Implemented")

    def get_header(self):
        "Get the header - First row. Subclasses should override this"
        raise GraphosException("Not Implemented")

    def get_first_column(self):
        """
        Get the first column. Generally would be the x axis.
        Subclasses should override this
        """
        raise GraphosException("Not Implemented")



from .base import BaseDataSource


class SimpleDataSource(BaseDataSource):
    def __init__(self, data):
        """
        : param data: :type list of lists
        Example usage:
            data = [
               ['Year', 'Sales', 'Expenses', 'Items Sold', 'Net Profit'],
               ['2004', 1000, 400, 100, 600],
               ['2005', 1170, 460, 120, 310],
               ['2006', 660, 1120, 50, -460],
               ['2007', 1030, 540, 100, 200],
            ]
            sd = SimpleDataSource(data)
        """
        self.data = data

    def get_data(self):
        return self.data

    def get_header(self):
        return self.data[0]

    def get_first_column(self):
        """
        Get the first column. Generally would be the x axis.
        : return: :type list of strings
        Example:
        For the example shown in __init__, it would return the following:
            ['2004', '2005', '2006', '2007']
        """
        data_not_header = self.data[1:]
        return [el[0] for el in data_not_header]
